#!/usr/bin/env python
# coding=UTF-8
#
# Copyright (C) 2010 Stefanos Harhalakis
#
# This file is part of wifieye.
#
# wifieye is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# wifieye is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with wifieye.  If not, see <http://www.gnu.org/licenses/>.
#
# Based on the idea of iwlistparse.py
# Hugo Chargois - 17 jan. 2010 - v.0.1
# Parses the output of iwlist scan into a table
# From http://bbs.archlinux.org/viewtopic.php?id=88967
# There is no code or logic similarity. It was just inspired by that.
# Everything is re-written from scratch

# $Id: scan.py 2265 2010-02-21 19:16:26Z v13 $

__version__ = "$Id: scan.py 2265 2010-02-21 19:16:26Z v13 $"

import pickle, subprocess, time
import sys

iface="wlan0"

# From /usr/include/wireless.h

enc_capa=[
    ('WPA',	0x00000001),	# IW_ENC_CAPA_WPA
    ('WPA2',	0x00000002),	# IW_ENC_CAPA_WPA2
    ('TKIP',	0x00000004),	# IW_ENC_CAPA_CIPHER_TKIP
    ('CCMP',	0x00000008),	# IW_ENC_CAPA_CIPHER_CCMP
    ]

auth_wpa=[
    ('DISABLED',    0x00000001),    # IW_AUTH_WPA_VERSION_DISABLED
    ('WPA',	    0x00000002),    # IW_AUTH_WPA_VERSION_WPA
    ('WPA2',	    0x00000004),    # IW_AUTH_WPA_VERSION_WPA2
    ]

auth_cipher=[
    ('NONE',	    0x00000001),    # IW_AUTH_CIPHER_NONE
    ('WEP40',	    0x00000002),    # IW_AUTH_CIPHER_WEP40
    ('TKIP',	    0x00000004),    # IW_AUTH_CIPHER_TKIP
    ('CCMP',	    0x00000008),    # IW_AUTH_CIPHER_CCMP
    ('WEP104',	    0x00000010),    # IW_AUTH_CIPHER_WEP104
    ]

auth_key_mgmt=[
    ('8021X',	    1),		    # IW_AUTH_KEY_MGMT_802_1X
    ('PSK',	    2),		    # IW_AUTH_KEY_MGMT_PSK
    ]

netwasdown=False

def match(line, st):
    """ Check if a line starts with this string, ingoring spaces """
    l=line.strip()
    if len(st)>len(l):
	return False

    if l[:len(st)]==st:
	return True

    return False

def first_pass(lines):
    """ Scan all lines and split the cells
    Return a list of cells"""
    buf=[]
    inbuf=False
    cells=[]
    for l in lines:
	if match(l, "Cell "):
	    if inbuf:
		cells.append(buf)
		buf=[]
	    inbuf=True
	elif inbuf and (len(l)==0 or not l[0].isspace()):
	    inbuf=False
	    cells.append(buf)
	    buf=[]
	if inbuf:
	    buf.append(l)

    if len(buf)>0:
	cells.append(buf)

    return(cells)

def parse_cell(cell):
    ret={
	'bssid':    '',
	'essid':    '',
	'mode':	    '',
	'quality':  {
	    'quality':	-1,
	    'noise':	-1,
	    'signal':	-1,
	    },
	'freq':	{
	    'm':    -1,
	    'e':    -1,
	    'chan': -1,
	    'freq': -1,
	    },
	'rates':    [],
	'enc':	    [],
	}

    lastenc=-1
    hasenc=False
    inbitrates=False
    for line in cell:
	l=line.strip()
	if match(line, "Cell "):
	    l2=l.split()
	    ret['bssid']=l2[4]
	elif match(line, 'ESSID:'):
	    l2=l.split(':')
	    ret['essid']=l2[1][1:-1]
	elif match(line, 'Channel:'):
	    l2=l.split(':')
	    ret['freq']['chan']=int(l2[1])
	elif match(line, 'Frequency:'):
	    l2=l.split()[0].split(':')
	    ret['freq']['freq']=int(float(l2[1])*1000)
	    ret['freq']['m']=ret['freq']['freq']
	    ret['freq']['e']=6
	elif match(line, 'Quality='):
	    l2=l[8:].split('  ')
	    ret['quality']={
		'quality':  int(l2[0].split('/')[0]),
		'signal':   int(l2[1].split(':')[1].split(' ')[0]),
		'noise':    int(l2[2].split('=')[1].split(' ')[0]),
		}
	elif match(line, 'Mode:'):
	    l2=l.split(':')
	    ret['mode']=l2[1]
	elif match(line, 'Encryption key:on'):
	    hasenc=True
	elif match(line, 'Encryption key:off'):
	    ret['enc'].append({'enc':'None', 'auth':['Open']})
	elif match(line, 'IE:'):
	    l2=l[4:]
	    enc={
		'enc':	    '',
		'gcipher':  '',
		'pciphers': [],
		'auth':	    [],
		}
	    if match(l2, 'IEEE 802.11i/WPA2 Version '):
		lastenc=len(ret['enc'])
		ver=l2[len('IEEE 802.11i/WPA2 Version '):]
		enc['enc']="WPA2 v"+ver
		ret['enc'].append(enc)
	    elif match(l2, 'WPA Version '):
		lastenc=len(ret['enc'])
		ver=l2[len('WPA Version '):]
		enc['enc']="WPA v"+ver
		ret['enc'].append(enc)
	elif lastenc>=0 and match(line, 'Group Cipher :'):
	    l2=l.split(':')[1].strip()
	    ret['enc'][lastenc]['gcipher']=l2
	elif lastenc>=0 and match(line, 'Pairwise Ciphers '):
	    l2=l.split(':')[1].strip().split()
	    ret['enc'][lastenc]['pciphers']=l2
	elif lastenc>=0 and match(line, 'Authentication Suites '):
	    l2=l.split(':')[1].strip().split()
	    ret['enc'][lastenc]['auth']=l2
	elif match(line, 'Bit Rates:'):
	    inbitrates=True
	    l2=l.split(':')[1].split(';')
	    for r in l2:
		r2=r.strip().split()
		ret['rates'].append(int(float(r2[0])*1000000))
	elif inbitrates:
	    if l.find('Mb')>=0:
		l2=l.split(';')
		for r in l2:
		    r2=r.strip().split()
		    ret['rates'].append(int(float(r2[0])*1000000))
	    else:
		inbitrates=False

    if hasenc and len(ret['enc'])==0:
	enc={'enc':  'WEP', 'auth': ['WEP']}
	ret['enc'].append(enc)

    return(ret)

def second_pass(cells):
    """ Parse cells """

    ret=[]
    for cell in cells:
	r=parse_cell(cell)
	ret.append(r)

    return(ret)

def _scan(iface):
    cmd=["/sbin/iwlist", iface, "scan"]
    sub=subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    r=sub.communicate()

    return(r)

def scan(iface):
    try:
	r=_scan(iface)
    except Exception, ex:
	r=('','')
	print "Exception in attempt #0"
	print ex

    # If there was no error (stderr is emptY) then return the data
    if len(r[1])==0:
	return(r[0])

    
    # If network was not down then this is an error
    if r[1].find('Network is down')==-1:
	print "Error:", r[1]
	return(None)

    # Else attempt to up&down the interface

    subprocess.call(["/sbin/ifconfig", iface, "up"])
    # Allow some time for interface to become ready
    time.sleep(0.2)

    try:
	r=_scan(iface)
    except Exception, ex:
	r=('', '')
	print "Exception in attempt #1"
	print ex

    subprocess.call(["/sbin/ifconfig", iface, "down"])

    if len(r[1])>0:
	return(None)

    return(r[0])

def show(dt):
    for i in dt:
	print "%s:" % (i,)
	for ii in dt[i]:
	    print "  %s:" % (ii,), dt[i][ii]
	print

def doit():
    r=scan(iface)
    if r!=None:
	cells=first_pass(r.splitlines())
	dt=second_pass(cells)
    else:
	dt={}

    dt2={}
    for i in dt:
	dt2[i['bssid']]=i

    if len(sys.argv)==1:
	print pickle.dumps(dt2)
    else:
	if sys.argv[1]=='1':
	    for i in dt2:
		print "%s: %s" % (i, dt2[i])
	else:
	    show(dt2)

doit()

# vim: set ts=8 sts=4 sw=4 noet formatoptions=r ai nocindent:

